// root -x 'FitOverlay.C("./ThermalSSHAroot")'
//Plots the fit over the data 
// Specify the nPlots species in the code

#if !defined(__CINT__) || defined(__MAKECINT__)
#include <Riostream.h>
#include <fstream>
#include <strstream>

#include <stdio.h>
#include <stdlib.h>
#include <TMath.h>

#include <TROOT.h>
#include <TFile.h>
#include <TH1.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <TMinuit.h>
#include <TGraph.h>

#endif

void RKLegendSetUp(TLegend *currentLegend,float currentTextSize);

void FitOverlay(Char_t *inFile){

TFile *f = new TFile(inFile);

if(!f || f->IsZombie())
  {
    cout << "Failed to load" << endl;
    return;
  }

//f->ls();
Int_t Marker_Style_combo[14] = {20,24,21,25,22,26,23,32,29,30};
Int_t Marker_Colors[7] = {95,95,226,2,1,1};


/*-------------*/ const Int_t nPlots = 6; /*-------------*/ //Number of Species

TH1F* hData[nPlots];
TH1F* hFit[nPlots];

for(Int_t i = 0; i < nPlots; i++ ){

    hData[i] = (TH1F*)f->Get(TString::Format("h_%d_0",i));
    hFit[i] = (TH1F*)f->Get(TString::Format("hFit_%d_0",i));

}

TCanvas *c1 = new TCanvas("c1","Canvas",10,10,1200,1800);
  c1->GetFrame()->SetFillColor(21);
  c1->GetFrame()->SetBorderSize(12); 
  c1->SetLogy();

TH2F *fhist = new TH2F("fhist","Combined Blast Wave Fit",20,0.2,2.5,20,1e-3,1e2);
  fhist->GetXaxis()->SetTitle("p_{T} (GeV/c) ");
  fhist->GetYaxis()->SetTitle(" #frac{1}{2#pi N_{evt}} #frac{1}{p_{T}} #frac{d^{2}N}{dp_{T}dy} (GeV/c)^{-2} ");
  fhist->GetXaxis()->CenterTitle();
  fhist->GetYaxis()->CenterTitle();
  fhist->SetTickLength(0.02,"x");
  fhist->SetTickLength(0.02,"y");
  fhist->SetTitleOffset(1.30,"y");
  fhist->SetTitleOffset(1.1,"x");
  fhist->GetYaxis()->SetNdivisions(5);
  fhist->Draw("");



//TString cLeg[6] = {"#pi^{-}", "#pi^{+}", "K^{-}","K^{+}","p","#bar{p}"};
TString cLeg[6] = {"#pi^{+}", "#pi^{-}", "K^{+}","K^{-}","p","#bar{p}"};
Double_t legP[4] = {0.8,0.83,0.95,0.95};

TLegend *myLegend = new TLegend(legP[0], legP[1], legP[2],legP[3]);
RKLegendSetUp(myLegend,0.04);

Double_t xmin[10] = {0.25,0.25,0.4,0.3,0.6,0.3};   // 3 GeV
Double_t xmax[10] = { 0.8,0.8,0.7,0.65,1.3,1.2};  // 3 GeV


//Double_t xmin = 0.6 ; //protons
//Double_t xmax = 1.2 ; // protons

//Double_t xmin[10] = {0.1,0.1,0.2,0.2,0.3,0.3};   // 62.4 GeV
//Double_t xmax[10] = { 0.7,0.7,0.7,0.7,1.2,1.2};  // 62.4 GeV

for ( Int_t i = 0; i < nPlots; i++ ) {
    
    hData[i]->SetMarkerStyle(Marker_Style_combo[i]);
    hData[i]->SetMarkerColor(Marker_Colors[i]);
    myLegend->AddEntry(hData[i],cLeg[i],"lepf");
    hFit[i]->GetXaxis()->SetRangeUser(xmin[i],xmax[i]);
}

myLegend->Draw();

TPaveText *pt = new TPaveText(.05,.1,.95,.8);
   //pt->AddText(" (#pi^{#pm},K^{#pm},p ) p_{T} Spectra");
   pt->AddText(" Au + Au #sqrt{S_{NN}} = 3 GeV; Centrality(0-5%)");
   pt->AddText("Midrapidity ([-0.05,0.05] CM frame, [-1.10,-1.00] lab frame ");
   pt->AddLine(.0,.5,1.,.5);
   pt->Draw();


for( Int_t i=0; i < nPlots; i++ ){

  hData[i]->Draw("SAME P");
  hFit[i]->Draw("SAME HIST L");
}


}



void RKLegendSetUp(TLegend *currentLegend=0,float currentTextSize=0.07){
    currentLegend->SetTextFont(42);
    currentLegend->SetBorderSize(0);
    currentLegend->SetFillStyle(0);
    currentLegend->SetFillColor(0);
    currentLegend->SetMargin(0.25);
    currentLegend->SetTextSize(currentTextSize);
    currentLegend->SetEntrySeparation(0.5);
    return;
}