//Combined Blast Wave Code  


#if !defined(__CINT__) || defined(__MAKECINT__)
#include <Riostream.h>
#include <fstream>
#include <strstream>

#include <stdio.h>
#include <stdlib.h>
#include <TMath.h>

#include <TROOT.h>
#include <TFile.h>
#include <TH1.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TLegend.h>
#include <TMinuit.h>
#include <TGraph.h>

#endif


//Maximum number of particle species
const unsigned int MaxNPart = 10;
//Maximum number of centrality bins
const unsigned int MaxNCent = 10;
const unsigned int MaxNBins = 100;
const unsigned int NPoints = 100;


int NPart;  // number of particle species to fit to BW
int NCent;  // number of centrality bins
int cent;   // Variable to loop over all centrality bins
            // varies from 0 to NCent
double m0[MaxNPart]; //mass of the particle species
double varsigma[MaxNPart];
//x is the pT, y is the yield and e the error
//x[particle type][centrality type][pT bin]

double x[MaxNPart][MaxNCent][MaxNBins];
double y[MaxNPart][MaxNCent][MaxNBins];
double e[MaxNPart][MaxNCent][MaxNBins];
double f[MaxNPart][MaxNCent][MaxNBins];
double chisquare[MaxNCent];

//The blastwave parameters
// Radial flow velocity BetaS
// Kinetic freezeout temperature Tfo
// Radial profile parameter (r/R)^N
//Along with their bin spacing

double mBetaS[MaxNCent];
double mTfo[MaxNCent];
double mN[MaxNCent];
double mdBetaS[MaxNCent];
double mdTfo[MaxNCent];
double mdN[MaxNCent];

int nDof[MaxNCent];
int nN;
int nC;

int nBins[MaxNPart];



int NMin = 3;


void Init();
double getYield(double betaS, double T, double kappa, double mT, double m0,double varsigma);
double Gaussian(double y, double sigma,double mean);
Double_t fRy(Double_t *x,Double_t *par);

double getAmplitude(int n, int c);
double getAmplitudeOld(int n, int c);
void fcn(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
void amp(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
void getdNdy(int n, int c);

double dNdyExp[MaxNPart][MaxNCent];
double edNdyExp[MaxNPart][MaxNCent];
double dNdyFit[MaxNPart][MaxNCent];
double edNdyFit[MaxNPart][MaxNCent];
double meanPt[MaxNPart][MaxNCent];
double emeanPt[MaxNPart][MaxNCent];
double emeanPtExp[MaxNPart][MaxNCent];
double frac[MaxNPart][MaxNCent];
double eFrac[MaxNPart][MaxNCent];

TROOT TROOT("rude", "rude");

TH1F* hTemp;
int main()
{
  Init();

  char buf[256];

  ostrstream *fileName = new ostrstream[10];

  ifstream mFile;
  istrstream *inp;

  FILE *fp;



  //Open the comand file to get the 
  //input variables, No. particle species
  //centrality bins, and the invariant yields vs. pT 
  //with errors

  
    mFile.open("bw_145_fit.txt");
     


  if (mFile.is_open() == 0)
    {
      cout << "  file not found! " << endl;
      return 0;
    } 

  //Read the number of particle species
  mFile.getline(buf, sizeof(buf)); NPart = atoi(buf);
  //Read the number of centrality bins
  mFile.getline(buf, sizeof(buf)); NCent = atoi(buf);
   
  //cout << NPart << endl;
  //cout << NCent << endl;

  mFile.getline(buf, sizeof(buf));


  //Read the mass of the particles
  //file the m0 values
  for (int n=0; n<NPart; n++)
    {
      mFile.getline(buf, sizeof(buf)); m0[n] = atof(buf);
      cout << m0[n] << endl;
    }

  cout << " Read all hadron mass "  << endl;

  //Read the sigma of their dN/dy distributions 
  //file the sigma values
  for (int n=0; n<NPart; n++)
    {
      mFile.getline(buf, sizeof(buf)); varsigma[n] = atof(buf);
      cout << varsigma[n] << endl;
    }

  cout << " Read all sigma values "  << endl;


  //Read fileNames
  for (int i=0; i<NPart; i++)
    {
      mFile.getline(buf, sizeof(buf));
      fileName[i] << buf << ends;
      cout << fileName[i].str() << endl;
    }
  mFile.close();

  cout << " Read all file names "  << endl;



  //START read data

  // First loop is over the particle species
  //say for example pion, Kaons, protons ....
  for (int n=0; n<NPart; n++)
    {
      char fname[256];
      sprintf(fname,"%s",fileName[n].str());
      cout << fname << endl;
      //ifstream mFile("./data/KPlusSpecMt1f_ALL_CuCu62.dat");
      //mFile.open(fileName[n].str());

      ifstream mFile(fileName[n].str());


      if (mFile.is_open() == 0) 
	{
	  cout << "file not found: " << fileName[n].str() << endl;
	  return 0;
	    } 
      int i=0;
      i = 0;
      cout << "  reading file " <<fileName[n].str() << endl;

      while (!mFile.eof())
	{
	  double pt;
	  mFile >> pt;
	  for (int c=0; c<NCent; c++)
	    {
	      x[n][c][i] = pt;
	      mFile >> y[n][c][i] >> e[n][c][i];
	      cout <<  x[n][c][i] << "  " << y[n][c][i] << "  " << e[n][c][i] << endl;
	    }
	  cout << i << endl;
	  i++;
	}
      nBins[n] = i-1;
      mFile.close();
    }

  /*
  File is read as following : i represents the pT bin
  0 for the 1st bin and so on .....
  first pT bin is read for the first particle type
  in all centrality bins then 2nd pT bin yield and err
  is read for  the first particle species for all centrality
  bins. The peripheral collisions are usually read first
  then we read for more central bins. After completeing 
  the first particle species we move to the second particle speices
  */


  //return 0;

  TGraph* tC[MaxNCent];

  //------------------------------------------//
  //We start with a loop over centrality bins
  //------------------------------------------//

  for (cent=0; cent<NCent; cent++)
    {

      cout<<"****** Starting centarlity bin "<<cent+1<<endl; 
      // +--------------------------+
      // | run MINUIT               |
      // +--------------------------+
      TMinuit *gMinuit = new TMinuit(3);  //initialize TMinuit with a maximum of 3 params
      gMinuit->SetMaxIterations(5000);
      gMinuit->SetFCN(fcn);
      
      Double_t arglist[10];
      Int_t ierflg = 0;
      
      // set err definition to 1-sigma
      arglist[0] = 1.;
      gMinuit->mnexcm("SET ERR", arglist ,1,ierflg);
      
      // Set starting values and step sizes for parameters
      //static Double_t vstart[] = {0.2, 0.2, 0.5};
      static Double_t vstart[] = {0.357, 0.07, 1.0};
      //      static Double_t vstart[] = {0.2, 0.23, 2.0};
      static Double_t step[] = {0.001, 0.01, 0.01};
      
      // Now ready for minimization step
      arglist[0] = 100; // number of function calls
      arglist[1] = 0.1; // tolerance
      
      TH1F* mdNdy[MaxNPart][MaxNCent];
      TH1F* mmeanPt[MaxNPart][MaxNCent];
     
      gMinuit->mnparm(0, "a1", vstart[0], step[0], 0.1, 0.7,ierflg);//0.0, 0.99 // This is beta
      gMinuit->mnparm(1, "a2", vstart[1], step[1], 0.06,0.12,ierflg);//0.05,0.30 //This is Tkin
      gMinuit->mnparm(2, "a3", vstart[2], step[2], 0.1,10.0,ierflg);//0.1,10  // This is n
    
      //gMinuit->FixParameter(2);
      
      gMinuit->mnexcm("MIGRAD", arglist ,2,ierflg); 
      gMinuit->mnexcm("MINOS", arglist ,2,ierflg);
      
      Double_t p0, p1, p2;
      Double_t dp0, dp1, dp2;
      Double_t dum1;
      Double_t dum2;
      TString *a1 = new TString("a1");
      TString *a2 = new TString("a2");
      TString *a3 = new TString("a3");
      gMinuit->mnpout(0, *a1, p0, dp0, dum1, dum2, ierflg);
      gMinuit->mnpout(1, *a2, p1, dp1, dum1, dum2, ierflg);
      gMinuit->mnpout(2, *a3, p2, dp2, dum1, dum2, ierflg);
      //cout << p0 << "  " << p1 << endl;
      
      mBetaS[cent] = p0;
      mTfo[cent] = p1;
      //mN[cent] = 0.0;
      mN[cent] = p2;
      mdBetaS[cent] = dp0;
      mdTfo[cent] = dp1;
      mdN[cent] = dp2; 
      
      // get variance
            
      cout << "Getting contour..." << endl;
      int np1 = 0; 
      int np2 = 1;
      double tx; 
      double ty; 
      
      TGraph* tG = (TGraph*) gMinuit->Contour(NPoints, np1, np2);
      if (gMinuit->GetStatus() != 0)
	{
	  cout << endl << endl;
	  cout << NPoints << " " << gMinuit->GetStatus()
	       << "  could not get nPoints on contour" << endl;
	  cout << endl << endl;
	} 
      else
	{
	  tC[cent] = new TGraph();
	  sprintf(buf, "tC_%d", cent);
	  tC[cent]->SetName(buf);
	  tC[cent]->SetTitle(buf);
	  char buf[80]; 
	  //------------------------------------------//
	  //Now we loop over a given particle species
	  //------------------------------------------//	 

	  for (int n=0; n<NPart; n++) 
	  { 

	    // get Mean 
	    for (int i=1; i<=hTemp->GetNbinsX(); i++)
	      { 
		double val =  hTemp->GetBinCenter(i);
		hTemp->SetBinContent(i, getYield(mBetaS[cent], mTfo[cent], mN[cent], val, m0[n],varsigma[n]));
	      }

	    for (int k=0; k<nBins[n]; k++)
	      { 
		double val =  x[n][cent][k];
		f[n][cent][k] = getYield(mBetaS[cent], mTfo[cent], mN[cent], val, m0[n],varsigma[n]);
	      } 

	    hTemp->Scale(getAmplitude(n, cent));
	    getdNdy(n, cent); 


	    double dNdyMean = dNdyFit[n][cent];
	    double ptMean = meanPt[n][cent];
	    
	    sprintf(buf, "dNdy_%d_%d", n, cent);
	    mdNdy[n][cent] = new TH1F(buf, buf, 2000, 0., 20.);
	    sprintf(buf, "mmeanPt_%d_%d", n, cent);
	    mmeanPt[n][cent] = new TH1F(buf, buf, 200, 0., 2.);
	    
	    for (int j=0; j<NPoints; j++)
	      {
		tG->GetPoint(j, tx, ty);
		for (int k=0; k<nBins[n]; k++)
		  {
		    double val =  x[n][cent][k];
		    f[n][cent][k] = getYield(tx, ty, mN[cent], val, m0[n],varsigma[n]);
		  }
		
		for (int i=1; i<=hTemp->GetNbinsX(); i++)
		  {
		    double val =  hTemp->GetBinCenter(i);
		    hTemp->SetBinContent(i, getYield(tx, ty, mN[cent], val, m0[n],varsigma[n]));
		  }
		hTemp->Scale(getAmplitude(n, cent));
		getdNdy(n, cent);
		mdNdy[n][cent]->Fill(dNdyFit[n][cent]);
		mmeanPt[n][cent]->Fill(meanPt[n][cent]);
		
		//cout << dNdyMean << "   " << ptMean << endl;
		if (fabs(dNdyMean - dNdyFit[n][cent]) > edNdyFit[n][cent])
		  {
		    edNdyFit[n][cent] = fabs(dNdyMean - dNdyFit[n][cent]);
		  }
		if (fabs(ptMean - meanPt[n][cent]) > emeanPt[n][cent])
		  {
		    emeanPt[n][cent] = fabs(ptMean - meanPt[n][cent]);
		  }
		//cout << "n:  " << mN[cent]<< endl;

		tC[cent]->SetPoint(j, tx * (2./(2.+mN[cent])), ty);
	      }
	    //edNdyFit[n][cent] = mdNdy[n][cent]->GetRMS();
	    //emeanPt[n][cent] = mmeanPt[n][cent]->GetRMS();
	  }
	}
      delete gMinuit;
      cout<<"****** Centrality bin "<<cent+1<<" done "<<endl; 
     
    } 
  // Print results
  Double_t amin,edm,errdef;
  Int_t nvpar,nparx,icstat;
   
   // +--------------------------+
   // | Plot results             |
   // +--------------------------+
			      
   // plot histograms
   TString* outName = new TString("ThermalSSHA.root");
   TFile histFile(outName->Data(), "RECREATE");

   TH1F* h[MaxNPart][MaxNCent];
   TH1F* hFit[MaxNPart][MaxNCent];

   TH1F* hBetaS = new TH1F("betaS", "betaS", 11, -0.5, 10.5);
   hBetaS->Sumw2();
   TH1F* hTfo   = new TH1F("Tfo", "Tfo", 11, -0.5, 10.5);
   hTfo->Sumw2();
   TH1F* hN     = new TH1F("n", "n", 11, -0.5, 10.5);
   hN->Sumw2();
   
   for (int n=0; n<NPart; n++)
     {
       for (int c=0; c<NCent; c++)
	 {
	   sprintf(buf, "h_%d_%d", n, c);
	   //	   h[n][c] = new TH1F(buf, buf, 60, 0., 3.); 
	   h[n][c] = new TH1F(buf, buf, 100, 0., 5.); 
	   h[n][c]->Sumw2();
	   sprintf(buf, "hFit_%d_%d", n, c);
	   hFit[n][c] = new TH1F(buf, buf, 100, 0., 5.); 
	   
	   for (int i=0; i<nBins[n]; i++)
	     {
	       int bin = h[n][c]->FindBin(x[n][c][i]);
	       h[n][c]->SetBinContent(bin, y[n][c][i]);
	       h[n][c]->SetBinError(bin, e[n][c][i]);
	     }
	   
	 
      for (int i=0; i<=nBins[n]; i++)
	     {
	       float val = x[n][c][i];
	       int bin = hFit[n][c]->FindBin(val);
	       hFit[n][c]->SetBinContent(bin, getYield(mBetaS[c], mTfo[c], mN[c], val, m0[n],varsigma[n]));
	     }
	   h[n][c]->Write();
	   
	   for (int i=0; i<nBins[n]; i++)
	     {
	       double val =  x[n][c][i];
	       f[n][c][i] = getYield(mBetaS[c], mTfo[c], mN[c], val, m0[n],varsigma[n]);
	     }
	   
	   hFit[n][c]->Scale(getAmplitude(n, c));
	   hFit[n][c]->Write();
	   

	  
	   hTemp->Reset();
	   hTemp->Add(hFit[n][c]);
	   getdNdy(n, c);
	 }
     }
   
   cout << "  beta" << endl;

   for (int c=0; c<NCent; c++){
       printf("%5.3f %5.3f\n", 2./(mN[c]+2.)*mBetaS[c], 2./(mN[c]+2.)*mdBetaS[c]);
     }


   cout << "Tfo" << endl;
   for (int c=0; c<NCent; c++)
     {
       printf("%6.3f %6.3f\n", mTfo[c], mdTfo[c]); 
     }
   cout << endl;
   
   cout << "n" << endl;
   for (int c=0; c<NCent; c++) 
     { 
       printf("%6.3f %6.3f\n", mN[c], mdN[c]);  
     } 
   cout << endl;
   
   for (int n=0; n<NPart; n++)
     {
       cout << "dN/dy, nParticle " << n << endl;
       for (int c=0; c<NCent; c++)
	 {
	   double err =  pow(edNdyExp[n][c],2) + pow(edNdyFit[n][c],2);
	   err = sqrt(err);
	   //cout << dNdyExp[n][c] << " +-  " << edNdyExp[n][c] << "   " << dNdyFit[n][c] << " +- " << edNdyFit[n][c] <<  "   " << dNdyFit[n][c] + dNdyExp[n][c] << " +- " << err << endl;
	   printf("%8.3f%8.3f\n", dNdyFit[n][c] + dNdyExp[n][c], err);
	 }
       cout << endl;
     }
   
   for (int n=0; n<NPart; n++)
     {
       cout << "<pt>, nParticle " << n << endl;
       for (int c=0; c<NCent; c++)
	 {
	   double err = pow(emeanPt[n][c]/meanPt[n][c],2) + pow(emeanPtExp[n][c]/meanPt[n][c],2);
	   err = sqrt(err);
	   printf("%6.3f %6.3f %6.3f %6.3f\n", meanPt[n][c], err*meanPt[n][c], emeanPt[n][c], emeanPtExp[n][c]);
	 }
       cout << endl;
     }

   for (int n=0; n<NPart; n++)
     {
       cout << "frac, nParticle " << n << endl;
       for (int c=0; c<NCent; c++)
	 {
	   printf("%6.3f %6.3f\n", frac[n][c], eFrac[n][c]);
	 }
       cout << endl;
     }
   
   cout << endl;
   for (int c=0; c<NCent; c++) 
     {
       cout << "XiSquare / nDof " << chisquare[c] << " / " << nDof[c] << endl;
     }


   // fit parameters
   for (int c=0; c<NCent; c++)
     {
       int bin = hBetaS->FindBin(c);
       hBetaS->SetBinContent(bin, 2./(mN[c]+2.)*mBetaS[c]);
       hTfo->SetBinContent(bin, mTfo[c]);
       hN->SetBinContent(bin, mN[c]);
       hBetaS->SetBinError(bin, mdBetaS[c]);
       hTfo->SetBinError(bin, mdTfo[c]);
       hN->SetBinError(bin, mdN[c]);
     }
   
   hBetaS->Write();
   hTfo->Write();
   hN->Write();
   
   histFile.Close();
   

   return 1;
}


double Gaussian(double y, double sigma,double mean){

 double f1 = pow((y-mean),2);
 double f2 = 2*pow(sigma,2);
 double f3 = -f1/f2;
 
 return TMath::Exp(f3);

}



Double_t fRy(Double_t *x,Double_t *par){

  //par[0] = Tkin
  //par[1] = betaS
  //par[2] = n
  //par[3] = mT
  //par[4] = sigma
  //par[5] = pT
  //x[0] = r
  //x[1] = y
  const double R = 10.;
  const double ycm = 1.045;
  double beta = par[1] * pow(x[0]/R, par[2]);
  double rho = TMath::ATanH(beta);
  
  return par[3] * x[0] * TMath::BesselI0(par[5] * TMath::SinH(rho) / par[0]) * TMath::CosH(x[1]) * TMath::Exp(-par[3] * TMath::CosH(x[1]) * TMath::CosH(rho)) * Gaussian(x[1],par[4],ycm);

}

double getYield(double betaS, double T, double kappa, double pT, double m0, double sigma){

    double mT = sqrt(pow(pT,2) + pow(m0,2));
    TF2 func2("fRy",fRy,0.,10.,-2.04,2.04,6);
    func2.SetParameter(0,T);
    func2.SetParameter(1,betaS);
    func2.SetParameter(2,kappa);
    func2.SetParameter(3,mT);
    func2.SetParameter(4,sigma);
    func2.SetParameter(5,pT);
    
    return func2.Integral(0.,10.,-2.04,2.04);
}




double getAmplitude(int n, int c)
{
  nN = n;
  cent = c;

  // +--------------------------+
  // | run MINUIT               |
  // +--------------------------+
   TMinuit *gMinuit = new TMinuit(1);  //initialize TMinuit with a maximum of 1 params
   gMinuit->SetMaxIterations(5000);
   gMinuit->SetFCN(amp);

   Double_t arglist[10];
   Int_t ierflg = 0;

   arglist[0] = 1.;
   gMinuit->mnexcm("SET ERR", arglist ,1,ierflg);

// Set starting values and step sizes for parameters
   static Double_t vstart[] = {10000.};
   static Double_t step[] = {1000.0};

// Now ready for minimization step
   arglist[0] = 500;
   arglist[1] = 0.5;

   gMinuit->mnparm(0, "a1", vstart[0], step[0], 0.0, 1.0e6,ierflg);
   
   gMinuit->mnexcm("MIGRAD", arglist ,2,ierflg); 
   //gMinuit->mnexcm("MINI", arglist ,2,ierflg);
   
   Double_t p0;
   Double_t dp0;
   Double_t dum1;
   Double_t dum2;
   TString *a1 = new TString("a1");
   gMinuit->mnpout(0, *a1, p0, dp0, dum1, dum2, ierflg);
   
   double kappa = p0;

   delete gMinuit; 
   return kappa;
}
   
double getAmplitudeOld(int n, int c)
{
  double SumA = 0.;
  double SumB = 0.;
  double Sum1 = 0.;
  double Sum2 = 0.;
  
  for (int i=0; i<nBins[n]; i++)
    {
      if (y[n][c][i] > 0.)
	{
	  Sum1 += y[n][c][i]*f[n][c][i] / pow(e[n][c][i],2);
	  Sum2 += pow(f[n][c][i] / e[n][c][i], 2);
	  SumA += y[n][c][i];// * x[n][c][i];
	  SumB += f[n][c][i];// * x[n][c][i];
	}
    }
  
  //double kappa = (SumB > 0.) ? SumA/SumB : 0.;
  double kappa = (Sum2>0.) ? Sum1/Sum2 : 0.;
  if (!(Sum2>0.)) 
    {
      //cout << "Shit!  " << c << endl;
      //cout << mBetaS[c] << "   " << mTfo[c] << "   " << mN[c] << "   " << m0[n] << endl;
      for (int i=0; i<nBins[n]; i++)
	{
	  //cout << x[n][c][i] << "   " << y[n][c][i] << "   " << f[n][c][i] << "   " << getYield(mBetaS[c], mTfo[c], mN[c], x[n][c][i], m0[n]) << endl;
	}
    }
  
  //cout << endl << endl;
  //cout << "Kappa, n " << kappa << "   " << n << endl;
  //cout << endl << endl;
  
   return kappa;
}

void fcn(Int_t &npar, Double_t *gin, Double_t &func, Double_t *par, Int_t iflag)
{

  int ndof = 0;

  //cout << iflag << endl;
  
  // get all historgrams
  for (int n=0; n<NPart; n++)
    {
      //cout << endl;
      for (int i=0; i<nBins[n]; i++)
	{
	  double b = par[0];
	  double Temp = par[1];
	  double kappa = par[2];
	  double val =  x[n][cent][i];
	  f[n][cent][i] = getYield(b, Temp, kappa, val, m0[n],varsigma[n]);
	  //cout <<  x[n][cent][i] << "  " << y[n][cent][i] << "  " << f[n][cent][i] << endl;
	}
    }
 
  // calc XiSquare
  Double_t chisq = 0;
  Double_t delta;
  
   for (int n=0; n<NPart; n++)
     {
       //cout << "Running blabla...  " << par[0] << "   " << par[1] << "   " << par[2] << endl;
       double fAmplitude = getAmplitude(n, cent);
       //cout << endl;
       //cout << n << "  Amplitude: " << fAmplitude << "   " <<  getAmplitudeOld(n, cent) << endl;
       //cout << endl;
    cout << "particle number? " << n << endl;
       for (int i=0; i<nBins[n]; i++){
	   if (e[n][cent][i] > 0.0)
	     {
	       ndof++;
         //cout << "particle number? " << n <<" i? " << i << endl;
	       //cout <<"Experimental Yields " << y[n][cent][i] << "   " << "Theoretical Yields with fAmp "<< fAmplitude*f[n][cent][i] << " "<< "Theoretical Yields without fAmp " << f[n][cent][i] <<endl;
         Double_t Ratio =  y[n][cent][i]/(fAmplitude*f[n][cent][i]);
         cout << y[n][cent][i] << " "<< fAmplitude*f[n][cent][i] << " Ratio =  " << Ratio << endl;
	       delta  = (y[n][cent][i] - fAmplitude*f[n][cent][i]) / e[n][cent][i];
	       chisq += delta*delta;
	     }
	 }
     }
   //cout << chisq << endl;
   func = chisq;

   nDof[cent] = ndof; 
   chisquare[cent] = chisq;
}

void amp(Int_t &npar, Double_t *gin, Double_t &func, Double_t *par, Int_t iflag)
{

  int n = nN;
  double chisq = 0.;
  for (int i=0; i<nBins[n]; i++)
    {
      if (e[n][cent][i] > 0.0)
	{
	  //cout <<  x[n][cent][i] << "  " << y[n][cent][i] << "  " << e[n][cent][i] << "  " << f[n][cent][i] << endl;
	  double delta = (y[n][cent][i] - par[0]*f[n][cent][i]) / e[n][cent][i];
	  chisq += delta*delta;
	}
    }
  func = chisq;
}

void getdNdy(int n, int c)
{
  float Sum1 = 0.;
  float Sum2 = 0.;
  float Sum3 = 0.;
  float SumExp = 0.;
  float eSumExp = 0.;
    
  float ptMin = 0.;
  float ptMax = 100.;

  float MeanPt = 0.;
  float errMeanPt = 0.;
  float wSum = 0.;

  bool flag = false;
  bool flag2 = false;
  double dx = x[n][c][1] - x[n][c][0];
    
  //For phi
  //  double dx[14] = {0.1,0.1,0.1,0.1,0.1,0.1,0.1};


  int bin1 = 0;
  int bin2 = 0;
  for (int i=0; i<nBins[n]; i++)
    {
      if (!(flag) && y[n][c][i] > 1.0e-10)
	{
	  ptMin = x[n][c][i] - dx/2.; 
	  // ptMin = x[n][c][i] - dx[i]/2.; 
	  flag = true;
	  bin1 = i;
	}
      if (!(flag2) && flag &&  y[n][c][i] < 1.0e-10)
	{
	  ptMax = x[n][c][i] - dx/2.; 
	  //ptMax = x[n][c][i] - dx[i]/2.; 
	  flag2 = true;
	  bin2 = i-1;
      }
      if (!(flag2) && flag &&  i==nBins[n]-1)
	{
	  ptMax =  x[n][c][i] + dx/2.; 
	  //ptMax =  x[n][c][i] + dx[i]/2.; 
	  bin2 = i;
	}
    }


  //cout << " I have got the pT max " << ptMax <<endl; 


  for (int i=bin1; i<=bin2; i++)
    {
      float pt =  x[n][c][i];
      // SumExp +=  y[n][c][i] * pt * dx[i]; 
       SumExp +=  y[n][c][i] * pt ; 
      eSumExp +=  e[n][c][i] > 0. ? pow(y[n][c][i]/e[n][c][i],2) : 0.; 
       float w = y[n][c][i] * pt * dx;
      //float w = y[n][c][i] * pt * dx[i];

      float ew =  e[n][c][i] / y[n][c][i];
      MeanPt += pt*w;
      errMeanPt += 1./pow(ew, 2);
      wSum += w;

	  //cout << pt << " pt and yield " << y[n][c][i] <<endl;
    }

  SumExp *= dx*2.*3.1415927; 
  //SumExp *= 2.*3.1415927; 
  eSumExp = SumExp / sqrt(eSumExp);

  emeanPtExp[n][c] = sqrt(1./errMeanPt)*MeanPt;
  dNdyExp[n][c] = SumExp;
  edNdyExp[n][c] = eSumExp;
 
  for (int i=1; i<=hTemp->GetNbinsX(); i++)
    {
      float pt = hTemp->GetBinCenter(i);
      if (pt < ptMin)
	{
	  Sum1 +=  hTemp->GetBinContent(i) * pt; 
	  float w = hTemp->GetBinContent(i) * pt * hTemp->GetBinWidth(i);
	  MeanPt += pt*w;
	  wSum += w;

	  //cout << pt << " pt and pT min " << ptMin << "  yield "  << hTemp->GetBinContent(i)<<endl;


	}
      else if (ptMin <= pt && pt < ptMax)
	{
	  Sum2 +=  hTemp->GetBinContent(i) * pt; 
	}
      else 
	{
	  Sum3 +=  hTemp->GetBinContent(i) * pt; 
	  float w = hTemp->GetBinContent(i) * pt * hTemp->GetBinWidth(i);
	  MeanPt += pt*w;
	  wSum += w;
	}
    }
  Sum1 *= hTemp->GetBinWidth(1)*2.*3.1415927; 
  Sum2 *= hTemp->GetBinWidth(1)*2.*3.1415927; 
  Sum3 *= hTemp->GetBinWidth(1)*2.*3.1415927; 

  meanPt[n][c] = MeanPt / wSum;
  emeanPtExp[n][c] /= wSum;
  dNdyFit[n][c] = Sum1 + Sum3;

  frac[n][c] = (Sum1 + SumExp + Sum3) / SumExp; 
  eFrac[n][c] = edNdyFit[n][c] / dNdyFit[n][c];

  //cout<< Sum1<<" "<< Sum2<< " "<<Sum3<<" "<< SumExp<<" "<<dNdyExp[n][c]+dNdyFit[n][c]<<endl;

}

void Init()
{
  hTemp = new TH1F("hTemp", "hTemp", 1000, 0., 5.);

  NPart = 0;
  NCent = 0;
  for (int n=0; n<MaxNPart; n++)
    {
      m0[n] = 0;
      nBins[n] = 0;
      for (int c=0; c<MaxNCent; c++)
	{
	  edNdyFit[n][c] = 0.;
	  emeanPt[n][c] = 0.;
	  for (int i=0; i<MaxNBins; i++)
	    {
	      x[n][c][i] = 0.;
	      y[n][c][i] = 0.;
	      e[n][c][i] = 0.;
	      f[n][c][i] = 0.;
	    }
	}
    }
}
